unit Cfgbde;

{ Define test to use c:\temp\idapi.cfg for testing }
{.$DEFINE TEST}

{ Create new BDE16 Alias by directly updating IDAPI.CFG. This allows
  also changing the path an alias points to without using BDECFG

  Thanks to Sean Walsh who kindly sent me sourcecode of his great work
  on deciphering IDAPI.CFG.

  This code is donated to the puiblic domain, do what ever you want with it.
  By using it you accept that I (the author) will not be responsible for
  any problems it causes.

  If you correct bugs, extend it or have any further suggestions, i would
  be grateful for your feedback to m-schumann@t-online.de! }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, INIFIles;

type
  TForm1 = class(TForm)
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const BakExtension='.CF~';


procedure Alias(name,path,defaultdriver,netdir:string);
{ Appends or modifies given alias in IDAPI.CFG without
  using the BDE }
var s,s1:string;
    inDBSection,
    aliasFound, Lastone: boolean;

var    FN: string;
       fin,fout: Textfile;

Function WindowsPath:String;
{ returns the directory of C:\WINDOWS\ on the current machine }
var d:integer; s:string;
  begin
    s:='';
    for d:=1 to 254 do s:=s+' ';;
    d:=getwindowsdirectory(@s[1],254);
    s[0]:=chr(d);
    if s[length(s)]<>'\' then s:=s+'\';
    result:=s;
  end;

function OpenCFG:boolean;
var ini: TINIFile;  FB:String; i:longint; c:char;
begin
  Result:=false;
  { get IDAPI-Config file location from WIN.INI
    !! works with Windows 3.x to Windows NT !! }
  INI:=TInifile.create(WindowsPath+'WIN.INI');
  FN:=INI.ReadString('IDAPI','CONFIGFILE01','');
  INI.Free;
  {$IFDEF TEST }
  FN:='c:\temp\idapi.cfg';
  {$ENDIF}
  { erase old IDAPI backup file if present }
  FB:=ChangeFileExt(FN,BakExtension);
  AssignFile(fin,FB);
  try
    Erase(fin);
    except { this may fail } end;
  { rename input file to backup file extension }
  try
    RenameFile(FN,FB);
    except exit; end;
  try
    { open old IDAPI.CFG for read }
    AssignFile(fin,FB);
    Reset(fin);
    { we look at 0300 as CRLF so we skip the first occurence }
    read(fin,c); read(fin,c);
    { open new IDAPI.CFG for write }
    AssignFile(fout,FN);
    Rewrite(fout);
    except exit; end;
  result:=true;
  end;


procedure XReadln(var S:String);
{ Special version of Readln, reads IDAPI.CFG-Line
  Every line in IDAPI.CFG starts with 0300. To make our
  live easier, we skip the first occurence and look
  at 0300 as CR/LF. When writing we automatically add
  0300 }
var c: char; EOR: boolean;
begin
  s:='';
  EOR:=false;
  while not EOF(fin) and not EOR do begin
      read(fin,c);
      if c=#3 then begin
        EOR:=true;
        read(fin,c); { skip 00 following 03 }
        end
      else s:=s+c;
   end;
  end;

procedure XWriteln(s:string);
begin
  write(Fout,#03#00);
  write(FOut,s);
  end;

begin
 { Make Random work properly }
 RandSeed:=GetTickCount;
 openCFG;
 inDBSection:=false;
 aliasFound:=false;
 while not EOF(fin) do begin
   XReadln(s);
   { Check for line containing NET DIR }
   if (pos(#1#0+'NET DIR',s)=1) and (netdir<>'') then begin
     { modify Net dir for Paradox }
     s:=#1#0+'NET DIR'+#0#4#0+netdir+#0;
     end;
   { Check for end of DATABASES section }
   if inDBSection and (pos(#2#0#2#0#2#0,s)>0) then begin
     { signal end of critical zone }
     inDBSection:=false;
     if not AliasFound then begin { add alias }
       { strip end of section sign = last 0200 }
       s:=copy(s,1,length(s)-2);
       XWriteln(s);
       { add the new alias at the end }
       XWriteln(#0#0+Uppercase(name)+#0#1#0);
       XWriteln(#0#0+'DB INFO'+#0#1#0);
       XWriteln(#0#1+'TYPE'+#0#4#0+'STANDARD'+#0);
       XWriteln(#0#1+'PATH'+#0#4#0+path+#0);
       s:=#0#1+'DEFAULT DRIVER'+#0#4#0+defaultdriver+#0#2#0#2#0#2#0;
       end;
    end;
   { Check for start of DATABASES section }
   if pos(#0+'DATABASES'+#0#1#0,s)>0 then inDBSection:=true;
   { While inside DATABASES section look for our alias }
   if inDBSection and (pos(#0+Uppercase(name)+#0#1#0,Uppercase(s))>0) then begin
     { we found it }
     XWriteln(s); { write header unchanged }
     { skip old alias's data }
     while not EOF(fin) and (pos(#2#0#2#0,s)=0) do XReadln(s);
     { if this is/was the last alias section, we must take care of this }
     LastOne:=pos(#2#0#2#0#2#0,s)>0;
     { insert the new alias data }
     XWriteln(#0#0+'DB INFO'+#0#1#0);
     XWriteln(#0#1+'TYPE'+#0#4#0+'STANDARD'+#0);
     XWriteln(#0#1+'PATH'+#0#4#0+path+#0);
     s:=#0#1+'DEFAULT DRIVER'+#0#4#0+defaultdriver+#0#2#0#2#0;
     if LastOne then s:=s+#2#0;
     AliasFound:=true;
     end;
   XWriteln(s);
   end;
 closefile(fin); closefile(fout);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
 Alias('TESTALIAS','C:\HELLO\THIS\WAS\ADDED\EASILY','PARADOX','F:\');
end;

end.


